﻿using System;
using System.Collections.Generic;
//using System.Linq;
using System.Text;
using Tao.OpenGl;
using System.Reflection;
using System.Drawing;

namespace SPS2
{
    public class Transition
    {
        protected bool done;
        public bool Done { get { return done; } set { done = value; } }
        protected bool autofinish = true;
        public bool AutoFinish { get { return autofinish; } }

        private DateTime m_Begin;
        protected long Elapsed_ms
        {
            get
            {
                return (long)(DateTime.Now - m_Begin).TotalMilliseconds;
            }
        }
        protected int TotalMsec;
        public virtual void Process()
        {
            if (autofinish)
            {
                if (Elapsed_ms > TotalMsec)
                {
                    done = true;
                }
            }

        }
        public virtual void Draw()
        {
        }
        protected int Width { get { return Display.Width; } }
        protected int Height { get { return Display.Height; } }

        public float ScaleFactor(float min, float max)
        {
            return ScaleFactor(0, TotalMsec, min, max);

        }
        public float ScaleFactor(int tick_min, int tick_max, float min, float max)
        {
            return ScaleFactor(tick_min, tick_max, min, max, Elapsed_ms);
        }

        public static float ScaleFactor(int tick_min, int tick_max, float min, float max, long elapsed_ms)
        {
            if (tick_max == tick_min)
            {
                throw new ArgumentException("tick_min == tick_max");
            }
            return (max - min) * (float)(elapsed_ms - tick_min) / (float)(tick_max - tick_min) + min;

        }

        private Display m_Display;
        public Display Display { get { return m_Display; } }
        public Transition(Display d, bool autofinish, int totalmsec)
        {
            m_Display = d;
            this.autofinish = autofinish;
            TotalMsec = totalmsec;

            done = false;
            m_Begin = DateTime.Now;
        }
    }

    public abstract class Flip : Transition
    {
        public class Flip_Parameter
        {
            public Scene Scene_before;
            public Scene Scene_after;
            public object Userdata;

            public Flip_Parameter()
            {
            }
        }

        private Flip_Parameter m_Parameter;
        protected Flip_Parameter Parameter { get { return m_Parameter; } }
        public Flip(Display d, Flip_Parameter p, bool autofinish, int totalmsec) : base(d, autofinish, totalmsec)
        {
            m_Parameter = p;
        }

        class Driver
        {
            public Assembly asm;
            public string typename;
            public Driver(Assembly a, string t)
            {
                asm = a;
                typename = t;
            }
        }
        private static Dictionary<string, Driver> m_Drivers;
        public static string[] DriverNames {
            get {
                string[] result = new string[m_Drivers.Keys.Count];
                int i = 0;
                foreach (string key in m_Drivers.Keys)
                {
                    result[i] = key;
                    i++;
                }
                return result;
            } 
        }
        static Flip()
        {
            m_Drivers = new Dictionary<string, Driver>();
            // TODO: DLL検索
            Assembly asm = Assembly.GetExecutingAssembly();
            foreach (Type t in asm.GetTypes())
            {
                if (t.BaseType == typeof(Flip))
                {
                    Flip f = asm.CreateInstance(t.FullName, false, BindingFlags.CreateInstance, null,
                        new object[] { null, null }, null, null) as Flip;
                    m_Drivers.Add(f.Name.ToLower(), new Driver(asm, t.FullName)) ;
                    System.Diagnostics.Debug.WriteLine(t.FullName);
                }
            }
        }
        public static Flip CreateInstance(string name, Display d, Flip_Parameter p)
        {
            if (name == null) return null;
            string key = name.ToLower();
            if (!m_Drivers.ContainsKey(key)) return null;

            Driver drv = m_Drivers[key];
            if (drv == null) return null;

            return drv.asm.CreateInstance(drv.typename, false, BindingFlags.CreateInstance, null,
                new object[] { d, p }, null, null) as Flip;
        }
        public abstract string Name{ get; }
        public abstract string Description { get; }


    }

    public class Flip_Palette : Flip
    {
        public override string Description
        {
            get { return "Forget Me Not - Palette -"; }
        }
        public override string Name
        {
            get { return "Palette"; }
        }
        public Flip_Palette(Display d, Flip_Parameter p) : base(d, p, true, 500) { }

        public override void Draw()
        {

            Parameter.Scene_after.Draw(Display);

            for (int i = 1; i <= 3; i++)
            {
                float alpha = ScaleFactor(1.0f, 0.0f);
                float ratio = 1.0f + ScaleFactor(0.0f, 1.0f) * i * 2;

                Gl.glColor4f(1.0f, 1.0f, 1.0f, alpha);
                Parameter.Scene_after.Draw(Display, delegate(DrawUnit u){
                    Gl.glPushMatrix();

                    Gl.glScalef(ratio, ratio, 1.0f);
                    Gl.glTranslatef(u.X - Width / 2.0f + u.Sprite.Width / 2.0f, u.Y - Height / 2.0f + u.Sprite.Height / 2.0f, 0.0f);

                    u.Sprite.Draw();
                    Gl.glPopMatrix();
                });
            }
        }
    }

    public class Flip_Random : Flip
    {
        public override string Description
        {
            get { return "Random"; }
        }
        public override string Name
        {
            get { return "Random"; }
        }

        private Flip m_Instance;
        public Flip_Random(Display d, Flip_Parameter p) : base(d, p, false, 1000) 
        {
            Random r = new Random();
            string[] names = Flip.DriverNames;
            string name = null;
            while(true){
                name = names[r.Next(0, names.Length)];
                if (name.ToLower() != this.Name.ToLower()) break;
            }
            System.Diagnostics.Debug.Assert(name != null);
            m_Instance = Flip.CreateInstance(name, d, p);
        }

        public override void Process()
        {
            m_Instance.Process();

            if (done == false)
            {
                done = m_Instance.Done;
            }
        }
        public override void Draw()
        {
            m_Instance.Draw();

        }
    }
    public class Flip_Checkin : Flip
    {
        public Flip_Checkin(Display d, Flip_Parameter p)
            : base(d, p, true, 1000)
        {
        }
        public override string Name
        {
            get { return "Checkin"; }
        }
        public override string Description
        {
            get { return "CHECKING YOU OUT!"; }
        }
        public override void Draw()
        {
            // 200msで移動と表示
            // それより早い＝表示済み
            // それより遅い＝まだ表示しない

            int display_ms = 200;

            // 表示範囲の決定
            int x_from = (int)((long)Display.Width * (Elapsed_ms - display_ms) / TotalMsec);
            int x_to = (int)((long)Display.Width * Elapsed_ms / TotalMsec);

            Parameter.Scene_after.Draw(Display, delegate(DrawUnit u)
            {
                // まだ表示しない
                if (u.X > x_to) return;

                Gl.glPushMatrix();

                Gl.glTranslatef(u.X - Width / 2.0f + u.Sprite.Width / 2.0f, u.Y - Height / 2.0f + u.Sprite.Height / 2.0f, 0.0f);
                if (u.X > x_from)
                {
                    float factor = (float)(u.X - x_from) / (float)(x_to - x_from);
                    Gl.glTranslatef(Width / 5.0f * factor, -Height / 1.0f * factor, 0.0f);
                    Gl.glScalef(3.0f - (1.0f - factor) * 2.0f, 3.0f - (1.0f - factor) * 2.0f, 1.0f);
                }

                u.Sprite.Draw();
                Gl.glPopMatrix();
            });
        }
    }
    public class Flip_Troopers : Flip
    {
        public Flip_Troopers(Display d, Flip_Parameter p) : base(d, p, true, 1000) { }
        public override string Name
        {
            get { return "Troopers"; }
        }
        public override string Description
        {
            get { return "beatmaniaIIDX 15 DJ Troopers' taste."; }
        }
        public override void Draw()
        {
            float alpha;
            // fade out
            //            alpha = (flip_param.elapsed_ms > 400 ? 1.7f - flip_param.elapsed_ms / 500.0f : 1.0f);
            alpha = ScaleFactor(1.0f, 0.0f);
            Gl.glColor4f(1.0f, 1.0f, 1.0f, alpha);

            Parameter.Scene_before.Draw(Display, delegate(DrawUnit u)
            {
                Gl.glPushMatrix();
                Gl.glTranslatef(u.X - Width / 2.0f + u.Sprite.Width / 2.0f, u.Y - Height / 2.0f + u.Sprite.Height / 2.0f, 0.0f);
                float ratio = ScaleFactor(1.0f, 20.0f);
                Gl.glScalef(ratio, ratio, 1.0f);
                u.Sprite.Draw();
                Gl.glPopMatrix();

            });

            if (Elapsed_ms > TotalMsec / 2)
            {
                alpha = ScaleFactor(TotalMsec / 2, TotalMsec, 0.5f, 1.0f);
                Gl.glColor4f(1.0f, 1.0f, 1.0f, alpha);

                Parameter.Scene_after.Draw(Display, delegate(DrawUnit u)
                {
                    Gl.glPushMatrix();
                    Gl.glTranslatef(u.X - Width / 2.0f + u.Sprite.Width / 2.0f, u.Y - Height / 2.0f + u.Sprite.Height / 2.0f, 0.0f);
                    float ratio = ScaleFactor(TotalMsec / 2, TotalMsec, 10.0f, 1.0f);
                    Gl.glScalef(ratio, ratio, 1.0f);
                    u.Sprite.Draw();
                    Gl.glPopMatrix();
                });
            }
        }
    }

    public class Flip_Fade : Flip
    {
        public Flip_Fade(Display d, Flip_Parameter p) : base(d, p, true, 1000) { }
        public override string Name
        {
            get { return "Fade"; }
        }
        public override string Description
        {
            get { return "Simple FadeIn/Out transition."; }
        }
        public override void Draw()
        {
            Gl.glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
            float alpha = ScaleFactor(0, TotalMsec / 2, 0.0f, 1.0f);

            if (Elapsed_ms < TotalMsec / 2)
            {
                // fade out
                Parameter.Scene_before.Draw(Display);

            }
            else
            {
                // fade in
                Parameter.Scene_after.Draw(Display);
                alpha = ScaleFactor(TotalMsec / 2, TotalMsec, 1.0f, 0.0f);
            }

            Color bgcolor = Display.Scene.page.setting.backcolor;
            Gl.glColor4f((float)bgcolor.R / 255.0f, (float)bgcolor.G / 255.0f, (float)bgcolor.B / 255.0f, alpha);

            Gl.glDisable(Gl.GL_TEXTURE_2D);
            Gl.glPushMatrix();
            Gl.glBegin(Gl.GL_QUADS);
            Gl.glVertex2f(-Width / 2.0f, -Height / 2.0f);
            Gl.glVertex2f(Width / 2.0f, -Height / 2.0f);
            Gl.glVertex2f(Width / 2.0f, Height / 2.0f);
            Gl.glVertex2f(-Width / 2.0f, Height / 2.0f);
            Gl.glEnd();
            Gl.glPopMatrix();
            Gl.glEnable(Gl.GL_TEXTURE_2D);
        }
    }

    public class Flip_CrossFader : Flip
    {
        public Flip_CrossFader(Display d, Flip_Parameter p) : base(d, p, true, 1000) { }

        public override string Name
        {
            get { return "CrossFader"; }
        }
        public override string Description
        {
            get { return "Simple CrossFading transition."; }
        }
        public override void Draw()
        {
            float alpha = ScaleFactor(1.0f, 0.0f);
            Gl.glColor4f(1.0f, 1.0f, 1.0f, alpha);
            // fade out
            Parameter.Scene_before.Draw(Display);

            // fade in
            Gl.glColor4f(1.0f, 1.0f, 1.0f, 1.0f - alpha);
            Parameter.Scene_after.Draw(Display);
        }
    }
    public class Fade_TypoH : Flip
    {
        public override string Description
        {
            get { return "Line Slide(Horizontal)."; }
        }
        public override string Name
        {
            get { return "TypoH"; }
        }
        public Fade_TypoH(Display d, Flip_Parameter p) : base(d, p, true, 1000) { }

        public override void Draw()
        {
            int slide;

            slide = (int)((long)Width * ScaleFactor(0.0f, 1.0f));
            Parameter.Scene_before.Draw(Display, delegate(DrawUnit u)
            {
                Gl.glPushMatrix();

                Gl.glTranslatef(u.X - Width / 2.0f + u.Sprite.Width / 2.0f, u.Y - Height / 2.0f + u.Sprite.Height / 2.0f, 0.0f);
                int direction = -(1 - (u.lineno % 2) * 2);
                Gl.glTranslatef(slide * direction, 0.0f, 0.0f);
                u.Sprite.Draw();
                Gl.glPopMatrix();
            });

            slide = (int)((long)Width * ScaleFactor(1.0f, 0.0f));
            Parameter.Scene_after.Draw(Display, delegate(DrawUnit u)
            {
                Gl.glPushMatrix();

                Gl.glTranslatef(u.X - Width / 2.0f + u.Sprite.Width / 2.0f, u.Y - Height / 2.0f + u.Sprite.Height / 2.0f, 0.0f);
                int direction = (1 - (u.lineno % 2) * 2);
                Gl.glTranslatef(slide * direction, 0.0f, 0.0f);
                u.Sprite.Draw();
                Gl.glPopMatrix();
            });
        }
    }
}
