﻿using System;
using System.Collections.Generic;
//using System.Linq;
using System.Text;

using System.Drawing;
using System.IO;
using Tao.Lua;
using Tao.OpenGl;

namespace SPS2
{

    public class Page
    {
        /// <summary>
        /// ページのセッティング
        /// </summary>
        public class Setting
        {
            public Bg bg;
            public Color backcolor;
            public Color forecolor;
            public SPS2.HORIZONTAL_ALIGN halign;
            public int hmargin;
            public string flip_driver;

            public Setting()
            {
                bg = new Bg();
            }
            public Setting(Setting from)
            {
                bg = from.bg;
                backcolor = from.backcolor;
                forecolor = from.forecolor;
                halign = from.halign;
                hmargin = from.hmargin;
                flip_driver = from.flip_driver;
            }
        }

        public List<DrawUnit> drawunits;
        public int pauselevel_max;
        public string[] source;
        public string title;


        public Setting setting;

        public Page()
        {
            setting = new Setting();
            drawunits = new List<DrawUnit>();
        }
        // ディスプレイリスト

        public void Add(DrawUnit u)
        {
            drawunits.Add(u);

        }

        private void LogWrite(RenderContext context, Log.Level l, string msg)
        {
            context.SPS.Log.Write(l, msg);
        }
        public void Parse(RenderContext context)
        {
            context.page_current = this;

            // TODO: dostringだとAccessVioliationExceptionが出てしまう

            //int status = Lua.luaL_dostring(context.lua, String.Join("\r\n", source));
            string path = "tmp.txt";
            File.WriteAllLines(path, source);
            int status = Lua.luaL_dofile(context.lua, path);
            if (status != 0)
            {
                LogWrite(context, Log.Level.Error, "Lua::Error::" + Lua.lua_tostring(context.lua, 1));
                Lua.lua_pop(context.lua, 1);
            }
        }
        private void DrawBG(Display d, float alpha)
        {
            Color bgcolor = d.Scene.page.setting.backcolor;
            Gl.glColor4f((float)bgcolor.R / 255.0f, (float)bgcolor.G / 255.0f, (float)bgcolor.B / 255.0f, (float)bgcolor.A / 255.0f * alpha);
            Gl.glDisable(Gl.GL_TEXTURE_2D);
            Gl.glPushMatrix();
            Gl.glBegin(Gl.GL_QUADS);
            Gl.glVertex2f(-d.Width / 2.0f, -d.Height / 2.0f);
            Gl.glVertex2f(d.Width / 2.0f, -d.Height / 2.0f);
            Gl.glVertex2f(d.Width / 2.0f, d.Height / 2.0f);
            Gl.glVertex2f(-d.Width / 2.0f, d.Height / 2.0f);
            Gl.glEnd();
            Gl.glPopMatrix();
            Gl.glEnable(Gl.GL_TEXTURE_2D);
        }
        public void Draw(Display d, int pauselevel)
        {
            foreach (DrawUnit u in drawunits)
            {
                if (u.pauselevel > pauselevel) continue;

                u.Draw(d);
            }
        }
        public void Draw(Display d, int pauselevel, DrawCallBack callback)
        {
            foreach (DrawUnit u in drawunits)
            {
                if (u.pauselevel > pauselevel) continue;

                callback(u);
            }
        }

        public delegate void DrawCallBack(DrawUnit u);
    }

    public class Scene
    {
        public Page page;
        public int pauselevel;

        public Scene() { }
        public Scene(Page page, int pauselevel)
        {
            this.page = page;
            this.pauselevel = pauselevel;
        }
        public Scene(Scene from) : this(from.page, from.pauselevel)
        {
        }
        public void Draw(Display d)
        {
            page.Draw(d, pauselevel);
        }
        public void Draw(Display d, Page.DrawCallBack c)
        {
            page.Draw(d, pauselevel, c);
        }
    }

}
