﻿using System;
using System.Collections.Generic;
//using System.Linq;
using System.Text;
using System.Drawing;
using System.IO;
using Tao.OpenGl;
using System.Text.RegularExpressions;

namespace SPS2
{
    public class RenderContext
    {
        Presentation presentation;
        string path;
        public IntPtr lua;
        public int pauselevel;
        public int lineno;

        public int pageno;

        public Page page_current;

        List<DrawUnit> line;

        bool in_linerender;
        bool in_pagerender;

        Point cursor;

        Page.Setting setting;

        Dictionary<string, Sprite> icons;

        private SPS2 m_Sps;
        public SPS2 SPS { get { return m_Sps; } }

        public Presentation Presentation { get { return presentation; } }
        public RenderContext(SPS2 sps, IntPtr L, string path, bool parse)
        {
            m_Sps = sps;
            lua = L;

            pauselevel = 0;
            lineno = 0;

            line = null;

            in_pagerender = false;
            in_linerender = false;

            icons = new Dictionary<string, Sprite>();
            setting = new Page.Setting();

            cursor.X = 0;
            cursor.Y = 0;

            setting.bg.style = Bg.BGSTYLE.TILE;
            setting.bg.sprite = null;
            setting.bg.alpha = 100.0;

            // XXX flip

            setting.halign = SPS2.HORIZONTAL_ALIGN.LEFT;
            setting.hmargin = 0;
            setting.backcolor = Color.Black;
            setting.forecolor = Color.White;


            this.path = path;

            presentation = new Presentation();
            presentation.ReadFile(this, SPS2.SYSTEM_CONFIG_PATH);
            presentation.ReadFile(this, path);
            if (parse)
            {
                presentation.ParseAll(this);
            }
        }



        private Display Display 
        {
            get
            {
                return m_Sps.Display;
            }
        }

        private void Add_DrawUnit(DrawUnit u)
        {
            if (!in_linerender)
            {
                page_current.Add(u);
                u.X = cursor.X;
                u.Y = cursor.Y;
                u.lineno = lineno;
                cursor.X += u.Sprite.Width;
            }
            else
            {
                line.Add(u);
            }
        }

        #region render関係(luaから呼び出し)

        public void Pause(double timer)
        {
            pauselevel++;
        }
        public void MoveCursor(double x, double y)
        {
            cursor.X = Display.RatioToPixelH(x);
            cursor.Y = Display.RatioToPixelV(y);
        }

        public void PutText(string text, string fontindex, SPS2.VERTICAL_ALIGN align)
        {
            SdlDotNet.Graphics.Font font = m_Sps.Fonts[fontindex];

            foreach (char ch in text)
            {
                DrawUnit u = new DrawUnit(this, font, page_current.setting.forecolor, ch);
                u.valign = align; // TODO: messy らしい（オリジナルのソースから）
                Add_DrawUnit(u);
            }
        }

        public void SetTitle(string title)
        {
            presentation.title = title;

        }


        public void BeginPage(string title)
        {
            cursor.X = 0;
            cursor.Y = 0;
            setting.hmargin = 0;

            pauselevel = 0;
            lineno = 0;

            in_pagerender = true;
            in_linerender = false;

            page_current.setting = new Page.Setting(setting);
            page_current.title = title;
        }

        private void LogWrite(Log.Level l, string msg)
        {
            m_Sps.Log.Write(l, msg);
        }
        public void DefFont(string name, string path)
        {
            m_Sps.FontPath[name] = path;
            LogWrite(Log.Level.Info, "fontpath(" + name + ") = " + path);
        }

        public void DefFontIndex(string id, string name, double point, int antialias, int idx)
        {
            int p = Display.RatioToPixel(point);

            string path = m_Sps.FontPath[name];

            SdlDotNet.Graphics.Font font = new SdlDotNet.Graphics.Font(path, p, idx);
            if (m_Sps.Fonts.ContainsKey(id))
            {
                // TODO: free
            }
            m_Sps.Fonts[id] = font;
        }

        public void ForeColor(string color)
        {
            // TODO: color
            Color c = Color_Parse(color);
            if (in_pagerender)
            {
                page_current.setting.forecolor = c;
            }
            else
            {
                setting.forecolor = c;
            }
        }
        public void BackColor(string color)
        {
            // TODO: color

            Color c = Color_Parse(color);
            if (in_pagerender)
            {
                page_current.setting.backcolor = c;
            }
            else
            {
                setting.backcolor = c;
            }
        }

        public void PutHBar(double thick)
        {
            int height = Display.RatioToPixelV(thick); 
            SdlDotNet.Graphics.Surface s = new SdlDotNet.Graphics.Surface(Display.Width - cursor.X, height);
            s.Fill(page_current.setting.forecolor);
            DrawUnit u = new DrawUnit(this, DrawUnit.DRAWUNIT_TYPE.PRIMITIVE, s);

            u.X = page_current.setting.hmargin;
            u.Y = cursor.Y;

            page_current.Add(u);

            cursor.X = page_current.setting.hmargin;
            cursor.Y += height;
        }
        public void MoveTo(double x, double y)
        {
            cursor.X += Display.RatioToPixelH(x);
            cursor.Y += Display.RatioToPixelV(y);
        }


        public void BeginLine(SPS2.HORIZONTAL_ALIGN halign)
        {
            in_linerender = true;
            page_current.setting.halign = halign;
            cursor.X = page_current.setting.hmargin;

            // TODO: lineのフラッシュ/free

            line = new List<DrawUnit>();

        }
        public void EndLine()
        {
            int x = cursor.X;
            int maxw = Display.Width;
            int maxh = 0;
            List<DrawUnit> linetmp = new List<DrawUnit>();
            foreach (DrawUnit u in line)
            {
                if (x + u.Sprite.Width >= maxw)
                {
                    // render 1line
                    Render(linetmp, maxh);
                    linetmp = new List<DrawUnit>();
                    
                    x = 0;
                    cursor.X = page_current.setting.hmargin;
                    cursor.Y += maxh;
                    lineno++;
                    maxh = 0;
                }
                linetmp.Add(u);
                x += u.Sprite.Width;
                if (maxh < u.Sprite.Height) maxh = u.Sprite.Height;
                u.lineno = lineno;
            }
            Render(linetmp, maxh);
            cursor.X = page_current.setting.hmargin; // XXX????
            cursor.Y += maxh;
            lineno++;
        }

        public void HMargin(double margin)
        {
            int m = Display.RatioToPixelH(margin);
            if (in_pagerender)
            {
                page_current.setting.hmargin = m;
            }
            else
            {
                setting.hmargin = m;
            }
        }


        public void EndPage()
        {
            page_current.pauselevel_max = pauselevel;
            in_pagerender = false;

        }

        public void FontStyle(string idx, string style)
        {
            SdlDotNet.Graphics.Font font = m_Sps.Fonts[idx];

            font.Style = TTF_Stye_Parse(style);
        }


        public void Flip_Function(string name)
        {
            if (in_pagerender)
            {
                page_current.setting.flip_driver = name;
            }
            else
            {
                setting.flip_driver = name;
            }


        }
        #endregion


        private static SdlDotNet.Graphics.Styles TTF_Stye_Parse(string style)
        {
            string s = style.ToLower();
            SdlDotNet.Graphics.Styles result = SdlDotNet.Graphics.Styles.None;
            if (s.Contains("b")) result |= SdlDotNet.Graphics.Styles.Bold;
            if (s.Contains("i")) result |= SdlDotNet.Graphics.Styles.Italic;
            if (s.Contains("u")) result |= SdlDotNet.Graphics.Styles.Underline;
            return result;

        }
        private static Color Color_Parse(string color)
        {
            Color result = Color.FromName(color);
            if (result.IsKnownColor) return result;
            Regex reg = new Regex("([0-9a-f][0-9a-f])([0-9a-f][0-9a-f])([0-9a-f][0-9a-f])");
            Match c = reg.Match(color.ToLower());
            if (c.Length >= 1)
            {
                result = Color.FromArgb(255,
                    Convert.ToInt32(c.Groups[1].Value, 16),
                    Convert.ToInt32(c.Groups[2].Value, 16),
                    Convert.ToInt32(c.Groups[3].Value, 16));
            }
            else
            {
                // TODO: 色がみつからない
            }

            return result;
        }
        private int GetTopLeftY(SPS2.VERTICAL_ALIGN valign, int h, int maxheight)
        {
            switch (valign)
            {
                case SPS2.VERTICAL_ALIGN.TOP:
                    return 0;
                case SPS2.VERTICAL_ALIGN.CENTER:
                    return (maxheight - h) / 2;
                case SPS2.VERTICAL_ALIGN.BOTTOM:
                    return maxheight - h;
                default:
                    System.Diagnostics.Debug.Assert(false, "Unknown Vertical Align");
                    break;
            }
            return 0;
        }
        private int GetTopLeftX(SPS2.HORIZONTAL_ALIGN halign, int w, int maxwidth)
        {
            switch (halign)
            {
                case SPS2.HORIZONTAL_ALIGN.LEFT:
                    return 0;
                case SPS2.HORIZONTAL_ALIGN.CENTER:
                    return (maxwidth - w) / 2;
                case SPS2.HORIZONTAL_ALIGN.RIGHT:
                    return maxwidth - w;
                default:
                    System.Diagnostics.Debug.Assert(false, "Unknown Horizontal Align");
                    break;
            }
            return 0;
        }


        private void Render(List<DrawUnit> line, int maxh)
        {
            int maxw = Display.Width;
            int totalw = 0;
            int x = cursor.X;

            foreach (DrawUnit u in line)
            {
                u.Y = cursor.Y + GetTopLeftY(u.valign, u.Sprite.Height, maxh);
                totalw += u.Sprite.Width;
            }
            x += GetTopLeftX(page_current.setting.halign, totalw, maxw - cursor.X);

            foreach (DrawUnit u in line)
            {
                u.X = x;
                x += u.Sprite.Width;
                page_current.Add(u);
            }
        }



        internal void DefIcon(string id, string name)
        {
            throw new NotImplementedException();
        }

        internal void PutIcon(string id, SPS2.VERTICAL_ALIGN valign, double w, double h)
        {
            int width = Display.RatioToPixel(w);
            int height = Display.RatioToPixel(h);

            return;
            throw new NotImplementedException();
        }

        internal void PutImage(string path, SPS2.VERTICAL_ALIGN valign, double w, double h)
        {
            int width = Display.RatioToPixel(w);
            int height = Display.RatioToPixel(h);

            return;
            throw new NotImplementedException();
        }



        internal void SetBg(string p, Bg.BGSTYLE bgstyle, double alpha)
        {
            return;
            throw new NotImplementedException();
        }
    }

}
