﻿using System;
using System.Collections.Generic;
//using System.Linq;
using System.Text;
using Tao.Lua;
using SdlDotNet.Graphics;
using SdlDotNet.Input;
using SdlDotNet.Core;
using Tao.OpenGl;
using System.Drawing;
using System.Threading;

namespace SPS2
{
    public class SPS2
    {
        public const string SYSTEM_CONFIG_PATH = "system.lua";
        Dictionary<string, string> fontpath;
        Dictionary<string, SdlDotNet.Graphics.Font> fonts;

        public Dictionary<string, string> FontPath { get { return fontpath; } set { fontpath = value; } }
        public Dictionary<string, SdlDotNet.Graphics.Font> Fonts { get { return fonts; } set { fonts = value; } }

        /// <summary>
        /// 垂直方向のアラインメント
        /// </summary>
        public enum VERTICAL_ALIGN
        {
            TOP,
            CENTER,
            BOTTOM,
        }
        /// <summary>
        /// 水平方向のアラインメント
        /// </summary>
        public enum HORIZONTAL_ALIGN
        {
            LEFT,
            CENTER,
            RIGHT,
        }
                
        public Log Log
        {
            get
            {
                return m_Log;
            }
        }

        public SPS2(Display d)
        {
            m_Log = new Log();

            display = d;

            Initialize();

            fontpath = new Dictionary<string, string>();
            fonts = new Dictionary<string, SdlDotNet.Graphics.Font>();


        }
        public void Load(string path)
        {
            IntPtr L = Lua.luaL_newstate();
            Lua.luaL_openlibs(L);
            LuaFunction.Register(L);

            RenderContext c = new RenderContext(this, L, path, false);
            context.Add(L, c);


            Presentation presentation = c.Presentation;

            presentation.ParseAll(c);
            display.presentation = presentation;

            display.Scene = new Scene();
            display.pageno = 0;

            display.AddScene(new Scene_Fps(display));
            Scene_MouseAction mouseaction = new Scene_MouseAction(display);
            display.AddScene(mouseaction);
            mouseaction.Focus();
        }

        private Log m_Log;

        private Display display;

        static Dictionary<IntPtr, RenderContext> context = new Dictionary<IntPtr, RenderContext>();
        public static RenderContext GetRenderContext(IntPtr L)
        {
            return context[L];
        }

        public Display Display
        {
            get { return display; }
        }



        #region Fields

        /// <summary>
        /// Lesson title
        /// </summary>
        public static string Title
        {
            get
            {
                return "SPS2 - ";
            }
        }

        #endregion Fields


        #region Lesson Setup
        /// <summary>
        /// Initializes methods common to all NeHe lessons
        /// </summary>
        protected void Initialize()
        {
            // Sets Window icon and title
            this.WindowAttributes();
            // Creates SDL.NET Surface to hold an OpenGL scene
            display.screen = Video.SetVideoMode(display.Width, display.Height, true, true, display.setting.fullscreen);
        }

        private bool m_Quit = false;
        public void MainLoop()
        {
            while (!m_Quit)
            {
                Tao.Sdl.Sdl.SDL_Event e;
                if(Tao.Sdl.Sdl.SDL_PollEvent(out e) > 0)
                {
                    switch (e.type)
                    {
                        case Tao.Sdl.Sdl.SDL_QUIT:
                            m_Quit = true;
                            return;
                        case Tao.Sdl.Sdl.SDL_KEYDOWN:
                            if(KeyDown(e.key)) continue;
                            break;
                    }
                    display.ProcessEvent(e);
                }
                display.Process();

                DrawGLScene();
                Video.GLSwapBuffers();
                Thread.Sleep(10);
            }
        }
        /// <summary>
        /// Sets Window icon and caption
        /// </summary>
        protected void WindowAttributes()
        {
            Video.WindowIcon();
            Video.WindowCaption = Title;
        }

        /// <summary>
        /// Resizes window
        /// </summary>
        public virtual void Reshape()
        {
            this.Reshape(100.0F);
        }

        /// <summary>
        /// Resizes window
        /// </summary>
        /// <param name="distance"></param>
        public virtual void Reshape(float distance)
        {
            // Reset The Current Viewport
            Gl.glViewport(0, 0, display.Width, display.Height);
            // Select The Projection Matrix
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            // Reset The Projection Matrix
            Gl.glLoadIdentity();
            // Calculate The Aspect Ratio Of The Window
            Glu.gluPerspective(45.0F, (display.Width / (float)display.Height), 0.1F, distance);
            // Select The Modelview Matrix
            Gl.glMatrixMode(Gl.GL_MODELVIEW);
            // Reset The Modelview Matrix
            Gl.glLoadIdentity();
        }

        /// <summary>
        /// Initializes the OpenGL system
        /// </summary>
        public virtual void InitGL()
        {
            // Enable Smooth Shading
            Gl.glShadeModel(Gl.GL_SMOOTH);
            // Black Background
            Gl.glClearColor(0.0F, 0.0F, 0.0F, 0.5F);
            // Depth Buffer Setup
            Gl.glClearDepth(1.0F);
            // Enables Depth Testing
            Gl.glEnable(Gl.GL_DEPTH_TEST);
            // The Type Of Depth Testing To Do
            Gl.glDepthFunc(Gl.GL_LEQUAL);
            // Really Nice Perspective Calculations
            Gl.glHint(Gl.GL_PERSPECTIVE_CORRECTION_HINT, Gl.GL_NICEST);
        }

        #endregion Lesson Setup

        #region void DrawGLScene

        /// <summary>
        /// Renders the scene
        /// </summary>
        protected virtual void DrawGLScene()
        {
            Color bgcolor = display.Scene.page.setting.backcolor;
            Gl.glClearColor((float)bgcolor.R / 255.0f, (float)bgcolor.G / 255.0f, (float)bgcolor.B / 255.0f, (float)bgcolor.A / 255.0f);

            // Clear Screen And Depth Buffer
            Gl.glClear((Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT));
            // Reset The Current Modelview Matrix
            Gl.glLoadIdentity();

            Enter2DMode();
            Gl.glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

            display.Draw();

            Leave2DMode();

        }
        protected void Enter2DMode()
        {
            Gl.glPushAttrib(Gl.GL_ENABLE_BIT);
            Gl.glDisable(Gl.GL_DEPTH_TEST);
            Gl.glDisable(Gl.GL_CULL_FACE);
            Gl.glEnable(Gl.GL_TEXTURE_2D);
            Gl.glEnable(Gl.GL_BLEND);
            Gl.glBlendFunc(Gl.GL_SRC_ALPHA, Gl.GL_ONE_MINUS_SRC_ALPHA);
            Gl.glViewport(0, 0, display.Width, display.Height);
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glPushMatrix();
            Gl.glLoadIdentity();

            //Gl.glOrtho(-display.Width / 2.0f, display.Width / 2.0f, display.Height / 2.0f, -display.Height / 2.0f, 0.0f, 10.0f);

            Gl.glFrustum(-(float)display.Width / 2.0f, (float)display.Width / 2.0f, (float)display.Height / 2.0f, -(float)display.Height / 2.0f, 1.0f, 10.0f);
            Gl.glTranslatef(0.0f, 0.0f, -3.0f);
            Gl.glScalef(3.0f, 3.0f, 1.0f);

            Gl.glMatrixMode(Gl.GL_MODELVIEW);
            Gl.glPushMatrix();
            Gl.glLoadIdentity();
            Gl.glTexEnvf(Gl.GL_TEXTURE_ENV, Gl.GL_TEXTURE_ENV_MODE, Gl.GL_MODULATE);
        }
        protected void Leave2DMode()
        {
            Gl.glMatrixMode(Gl.GL_MODELVIEW);
            Gl.glPopMatrix();
            Gl.glMatrixMode(Gl.GL_PROJECTION);
            Gl.glPopMatrix();
            Gl.glPopAttrib();
        }

        #endregion void DrawGLScene

        #region Event Handlers

        private bool KeyDown(Tao.Sdl.Sdl.SDL_KeyboardEvent e)
        {
            switch (e.keysym.sym)
            {
                case Tao.Sdl.Sdl.SDLK_ESCAPE :
                    // Will stop the app loop
                    m_Quit = true;
                    return true;
                case Tao.Sdl.Sdl.SDLK_F1:
                    // Toggle fullscreen
                    if ((display.setting.fullscreen))
                    {
                        display.screen = Video.SetVideoMode(display.Width, display.Height, true, true, true);
                        this.WindowAttributes();
                    }
                    else
                    {
                        display.screen = Video.SetVideoMode(display.Width, display.Height, true, true);
                    }
                    Reshape();
                    return true;
            }
            return false;
        }

        #endregion Event Handlers



    }
}
