=begin

パラグラフは字下げ、字下げレベルは見出しレベルで変化する。
テキスト空のパラグラフは行下げに使う。下げ幅は見出しレベルで変化

箇条書も同様。見出しレベル、ネストレベルに応じて、字下げ、アイコンが変化
番号つき箇条書サポート


見出しは、箇条書 + 見出しレベル増加
1 はページ変更、タイトルを兼ねる

リンク類はすべて無視(wikiname/link)

整形済テキストは等幅フォントで扱う(?)

強調1レベルは斜体、2レベルはボールド

水平線サポート

プラグイン呼び出し
{{sps_eval()}}は、引数の中身をそのまま出力する


いずれはスタイルファイルか何かで扱いたいねえ

$Id: hiki2lua.rb,v 1.3 2004/05/01 02:42:37 zinnia Exp $

=end
$KCODE = 'e'

require 'parser.rb'
$use_plugin = true

class LUAFormatter
  MAP = Hash::new
  def initialize(t)
    @tokens = t
  end
  def dump
    result = ''

    @tokens.each do |token|
      result += "#{token[:e]}: #{token[:s]}\n"
    end
    return result
  end
  def to_s
    result = "sps_include(\"default.lua\")\n"
    string = ''
    pageno = 0
    headinglevel = 0
    hmargin = [3, 5, 8, 11, 14]
    vmargin = [2, 2, 2, 1, 1]
    headtag = ['○', '◇', '▽', '＋', '・']

    @tokens.each do |token|
      case token[:e]
      when :normal_text, :wikiname, :reference
	string += token[:s].to_s.chomp
      when :p_open
	result += "sps_beginline()\n" # XXX align
	string = ''
      when :p_close
	result += "sps_puttext(\"#{string}\", #{headinglevel})\n" + 
	  "sps_endline()\n" +
	  "sps_moveto(0, #{vmargin[headinglevel]})\n"
      when :heading1_open
	if pageno > 0 then
	  result += "sps_endpage()\n"
	end
	string = ''
	pageno += 1
      when :heading1_close
	result += "sps_beginpage(\"#{string}\")\n" +
	  "sps_beginline()\n" +
	  "sps_puttext(\"#{string}\", 0)\n" + 
	  "sps_endline()\n" +
	  "sps_moveto(0, 1)\nsps_puthbar(1)\n" + 
	  "sps_hmargin(#{hmargin[0]})\n" +
	  "sps_moveto(0, #{vmargin[headinglevel]})\n"
	headinglevel = 1
      when :heading2_open
	headinglevel = 2
	string = ''
      when :heading3_open
	headinglevel = 3
	string = ''
      when :ordered_list_open
      when :unordered_list_open
      when :listitem_open
	string = ''
      when :listitem_close
	result += "sps_beginline()\n" +
	  "sps_puttext(\"・#{string}\", #{headinglevel})\n" +
	  "sps_endline()\n" +
	  "sps_moveto(0, #{vmargin[headinglevel]})\n"
      when :heading2_close, :heading3_close
	result +=
	  "sps_moveto(0, #{vmargin[headinglevel - 1]})\n" + 
	  "sps_hmargin(#{hmargin[headinglevel - 1]})\n" +
	  "sps_beginline()\n" +
	  "sps_puttext(\"#{headtag[headinglevel - 2]}#{string}\", #{headinglevel - 1})\n" +
	  "sps_endline()\n" + 
	  "sps_moveto(0, #{vmargin[headinglevel]})\n" +
	  "sps_hmargin(#{hmargin[headinglevel - 1]})\n"
      when :plugin
	case token[:method]
#	when 'sps_eval'
#	  result += token[:param] + "\n"
	when 'sps_include'
	  result += "sps_include(\"#{token[:param]}\")\n"
	when 'sps_moveto'
	  result += "sps_moveto(#{token[:param]})\n"
	when 'sps_pause'
	  result += "sps_pause()\n"
	when 'sps_flip_function'
	  result += "sps_flip_function(\"#{token[:param]}\")\n"
	when 'sps_putimage'
	  result += "sps_putimage(#{token[:param]})\n"
	when 'sps_bg'
	  result += "sps_bg(#{token[:param]})\n"
	end
      end
    end
    if result != '' then
      result += "sps_endpage()\n"
    end
    return result
  end
end

path = ARGV.shift
dump = ARGV.shift
if path != nil then
  open(path) do |p|
    parser = Hiki::Parser.new()

	lines = p.readlines
	#lines.each{|l| l.chomp!}
	#lines.each {|l| print "AAAA: '" + l + "'\n" }
    tokens = parser.parse(lines)
    lf = LUAFormatter.new(tokens)
    if dump != nil then
      print lf.dump
    else
      print "-- Auto generated by hiki2lua.rb\n" + lf.to_s
    end
  end
else
  STDERR.print "usage: hiki2lua.rb [input] [[dump]]\n"
end

